'''
Energy Efficiency Targeting Tool Copyright (c) 2018, The Regents of the University of California, 
through Lawrence Berkeley National Laboratory (subject to receipt of any required approvals from the U.S. Dept. of Energy). 
All rights reserved.

If you have questions about your rights to use or distribute this software, 
please contact Berkeley Lab's Intellectual Property Office at  IPO@lbl.gov.

NOTICE.  This Software was developed under funding from the U.S. Department of Energy 
and the U.S. Government consequently retains certain rights. As such, the U.S. Government 
has been granted for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
worldwide license in the Software to reproduce, distribute copies to the public, 
prepare derivative works, and perform publicly and display publicly, and to permit other to do so. 
'''

import pandas as pd
import numpy as np
import copy

from . import constants

# Modified by Han Li on 2018-7-31 for the open source tool
class LEAN_FIMs:
    def __init__(self, df_assessment, utility_type):
        self.df_assessment = copy.deepcopy(df_assessment)
        self.utility_type = utility_type
        self.utility_type_str = 'Electricity' if (utility_type == 1) else 'Fossil Fuel'
        self.base = self.df_assessment.site_coefficients.beta_base
        self.cdd = self.df_assessment.site_coefficients.beta_cdd
        self.betc = self.df_assessment.site_coefficients.beta_betc
        self.hdd = self.df_assessment.site_coefficients.beta_hdd
        self.beth = self.df_assessment.site_coefficients.beta_beth

        # Adapt the model coefficients to the LEAN_FIMs module
        if self.betc == self.beth:
            if self.cdd == 0:
                self.betc, self.cdd  = np.nan, np.nan
            if self.hdd == 0:
                self.beth, self.hdd = np.nan, np.nan

        self.benchmark_medians = self.df_assessment.beta_median
        self.benchmark_stdevs = self.df_assessment.beta_standard_deviation
        self.site_coeffs = [self.base, self.cdd, self.betc, self.hdd, self.beth]

    # self.utility_type: electricity = 1, fossil fuel = 2
    # target_level: conservative = 1, nominal = 2, aggressive = 3
    def set_targets(self, target_level):

        self.n = len(self.site_coeffs)
        self.targets = np.zeros(self.n)

        # print(self.n)
        print('---------------------------------------------------------------')
        print(self.utility_type_str)
        print('Site Coefficients:')
        print(self.site_coeffs)

        if target_level == 1:
            for i in range(0, self.n):
                beta_name = self.df_assessment.index[i]
                if np.isnan(self.site_coeffs[i]):
                    self.targets[i] = np.nan
                elif beta_name == 'beta_betc': # cooling change-point
                    # if coefficient is cooling change-point coefficient => the higher the better => get max value
                    self.targets[i] = max(self.benchmark_medians.iloc[i] - self.benchmark_stdevs.iloc[i], self.site_coeffs[i])
                else:
                    # for other coefficients => the smaller the better => get min value
                    # however for the heating slope coefficient whose the value is negative => the higher the better
                    # if the heating slope median sign is flipped
                    self.targets[i] = min(self.benchmark_medians.iloc[i] + self.benchmark_stdevs.iloc[i], self.site_coeffs[i]) # original code

        elif target_level == 2:
            for i in range(0, self.n):
                beta_name = self.df_assessment.index[i]
                if np.isnan(self.site_coeffs[i]):
                    self.targets[i] = np.nan
                else:
                    if beta_name == 'beta_betc':
                        self.targets[i] = max(self.benchmark_medians.iloc[i], self.site_coeffs[i])
                    else:
                        self.targets[i] = min(self.benchmark_medians.iloc[i], self.site_coeffs[i]) # original code

        elif target_level == 3:
            for i in range(0, self.n):
                beta_name = self.df_assessment.index[i]
                if np.isnan(self.site_coeffs[i]):
                    self.targets[i] = np.nan
                else:
                    if beta_name == 'beta_betc':
                        self.targets[i] = max(self.benchmark_medians.iloc[i] + (0.5 * self.benchmark_stdevs.iloc[i]), self.site_coeffs[i])
                    else:
                        self.targets[i] = min(self.benchmark_medians.iloc[i] - (0.5 * self.benchmark_stdevs.iloc[i]), self.site_coeffs[i]) # original code

        self.base_targ = self.targets[0]
        self.cdd_targ = self.targets[1]
        self.betc_targ = self.targets[2]
        self.hdd_targ = self.targets[3]
        self.beth_targ = self.targets[4]

        print('Target Coefficients:')
        print(self.targets)
        print('---------------------------------------------------------------')

    def FIM_recommendations(self, save_file=True):
        column = ['FIM Recommendations']
        rows = ['Increase Cooling Setpoints', 'Decrease Heating Setpoints',
                'Reduce Equipment Schedules', 'Ensure Adequate Ventilation Rate',
                'Eliminate Electric Heating', 'Decrease Infiltration',
                'Reduce Lighting Load', 'Reduce Plug Loads', 'Add/Fix Economizers',
                'Increase Cooling System Efficiency',
                'Increase Heating System Efficiency', 'Add Wall/Ceiling Insulation',
                'Upgrade Windows', 'Check Fossil Baseload']
        self.FIM_table = pd.DataFrame(index=rows, columns=column)

        ovrd_thres = 1
        ovrd_val = 0.001

        # Increase Cooling Setpoint (indicated by low betc)
        setpoint_recommendation = False  # used for scheduling measure
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.2
        if (self.betc_targ - self.betc) >= (threshold * self.betc_targ):
            self.FIM_table.loc['Increase Cooling Setpoints'] = 'X'
            setpoint_recommendation = True

        # Decrease Heating Setpoint (indicated by high beth)
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.2
        if (self.beth - self.beth_targ) >= (threshold * self.beth_targ):
            self.FIM_table.loc['Decrease Heating Setpoints'] = 'X'
            setpoint_recommendation = True

        # Tighten Schedules (indicated by high electric baseload)
        """
        9/20/13 - Added logic to also recommend schedules if recommending
        increasing cooling setpoints or decreasing heating setpoints. A
        building's break-even temperature are affected by the average building
        temperatures (i.e., occupied & unoccupied). That means that adjusting
        schedules for switching between an occupied and unoccupied setpoint
        will also change the average building temperatures.
        """
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.001

        if self.utility_type == 1 and self.base > 0 and (self.base - self.base_targ) >= (threshold * self.base_targ):
            self.FIM_table.loc['Reduce Equipment Schedules'] = 'X'
        elif setpoint_recommendation:
            self.FIM_table.loc['Reduce Equipment Schedules'] = 'X'

        # Decrease Ventilation (indicated by two of the following three: high cdd, high hdd, high beth)
        count = 0
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.1
        if self.cdd > 0 and (self.cdd - self.cdd_targ) >= (threshold * self.cdd_targ):
            count = count + 1
        if self.hdd > 0 and (self.hdd - self.hdd_targ) >= (threshold * self.hdd_targ):
            count = count + 1
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.2
        if (self.beth - self.beth_targ) >= (threshold * self.beth_targ):
            count = count + 1
        if count >= 2:
            # self.FIM_table.loc['Decrease Ventilation'] = 'X' # old version
            self.FIM_table.loc['Ensure Adequate Ventilation Rate'] = 'X'

        # Eliminate Any Electric Heating
        """
        Before 9/20/2013, electric heating indicated by electric fuel and HDD>0.
        Added threshold to improve diagnostic and reduce mis-diagnosis of electric
        heating. It appears that both electric and fossil heating has beta_hdd's
        around 0.04 kWh/m2 while electric beta_hdd's without electric heating
        are around 0.004 kWh/m2.
        """
        electric_htg_threshold = 0.01
        if self.utility_type == 1 and self.hdd > electric_htg_threshold:
            self.FIM_table.loc['Eliminate Electric Heating'] = 'X'

        # Decrease Infiltration (indicated by two of the following three: high cdd, high hdd, high beth)
        count = 0
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.1
        if self.cdd > 0 and (self.cdd - self.cdd_targ) >= (threshold * self.cdd_targ):
            count = count + 1
        if self.hdd > 0 and (self.hdd - self.hdd_targ) >= (threshold * self.hdd_targ):
            count = count + 1
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.2
        if (self.beth - self.beth_targ) >= (threshold * self.beth_targ):
            count = count + 1
        if count >= 2:
            self.FIM_table.loc['Decrease Infiltration'] = 'X'

        # Reduce Lighting Load (indicated by high baseload)
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.001
        if self.utility_type == 1 and self.base > 0 and (self.base - self.base_targ) >= (threshold * self.base_targ):
            self.FIM_table.loc['Reduce Lighting Load'] = 'X'

        # Reduce Plug Load (indicated by high baseload)
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.001
        if self.utility_type == 1 and self.base > 0 and (self.base - self.base_targ) >= (threshold * self.base_targ):
            self.FIM_table.loc['Reduce Plug Loads'] = 'X'

        # Add/Fix Economizers (indicated by low betc)
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.2
        if (self.betc_targ - self.betc) >= (threshold * self.betc_targ):
            self.FIM_table.loc['Add/Fix Economizers'] = 'X'

        # Increase Cooling Efficiency (indicated by high cdd)
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.1
        if self.cdd > 0 and (self.cdd - self.cdd_targ) >= (threshold * self.cdd_targ):
            self.FIM_table.loc['Increase Cooling System Efficiency'] = 'X'

        # Increase Heating Efficiency (indicated by high hdd)
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.1
        if self.hdd > 0 and (self.hdd - self.hdd_targ) >= (threshold * self.hdd_targ):
            # self.FIM_table.loc['Increase Heating System Efficiency'] = 'X' # original version
            self.FIM_table.loc['Increase Heating System Efficiency'] = 'X' # original version

        # Add Wall/Ceiling Insulation (indicated by two of the following three: high cdd, high hdd, high beth)
        count = 0
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.1
        if self.cdd > 0 and (self.cdd - self.cdd_targ) >= (threshold * self.cdd_targ):
            count = count + 1
        if self.hdd > 0 and (self.hdd - self.hdd_targ) >= (threshold * self.hdd_targ):
            count = count + 1
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.2
        if (self.beth - self.beth_targ) >= (threshold * self.beth_targ):
            count = count + 1
        if count >= 2:
            self.FIM_table.loc['Add Wall/Ceiling Insulation'] = 'X'

        # Upgrade Windows (indicated by all of the following: high cdd, low betc, high hdd)
        count = 0
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.1
        if self.cdd > 0 and (self.cdd - self.cdd_targ) >= (threshold * self.cdd_targ):
            count = count + 1
        if self.hdd > 0 and (self.hdd - self.hdd_targ) >= (threshold * self.hdd_targ):
            count = count + 1
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.2
        if (self.betc_targ - self.betc) >= (threshold * self.betc_targ):
            count = count + 1
        if count == 3:
            self.FIM_table.loc['Upgrade Windows'] = 'X'

        # Check Excessive Fossil Fuel Baseload
        if ovrd_thres:
            threshold = ovrd_val
        else:
            threshold = 0.001
        if self.utility_type == 2 and self.base > 0 and (self.base - self.base_targ) >= (threshold * self.base_targ):
            self.FIM_table.loc['Check Fossil Baseload'] = 'X'
        # if(save_file): self.FIM_table.to_csv(self.utility_type_str + " FIM_recommendations.csv")
        return self.FIM_table

    def savings_coefficients(self, save_file=True):
        columns = ['coefficients', 'original_coefficients', 'savings_coefficients']
        rows = ['beta_base', 'beta_cdd', 'beta_betc', 'beta_hdd', 'beta_beth']
        self.coeff_out = pd.DataFrame(columns=columns)
        for i in range(0, len(rows)):
            self.coeff_out.loc[i, columns[0]] = rows[i]
            self.coeff_out.loc[i, columns[1]] = self.site_coeffs[i]
            if(self.site_coeffs[i] > self.targets[i]):
                if(i==2):
                    self.coeff_out.loc[i, columns[2]] = self.site_coeffs[i]
                else:
                    self.coeff_out.loc[i, columns[2]] = self.targets[i]
            else:
                if(i==2):
                    self.coeff_out.loc[i, columns[2]] = self.targets[i]
                else:
                    self.coeff_out.loc[i, columns[2]] = self.site_coeffs[i]
        self.coeff_out = self.coeff_out.set_index(columns[0])
        # if(save_file): self.coeff_out.to_csv(self.utility_type_str + " Coeffs_out.csv")
        return self.coeff_out


if __name__ == "__main__":
    test_site_coeffs = [0, 2, 3, 0, 0]
    df_assessment = constants.Constants.df_sample_benchmark_stats
    df_assessment["site_coefficients"] = test_site_coeffs

    # print(df_assessment)

    example_FIM_analysis = LEAN_FIMs(df_assessment, 1)  # (electricity = 1, fossil fuel = 2)
    example_FIM_analysis.set_targets(1)  # conservative = 1, nominal = 2, aggressive = 3
    example_FIM_analysis.FIM_recommendations()
    example_FIM_analysis.savings_coefficients()



import os
from pathlib import Path


noaa_weather_station_list_file = os.path.abspath(
    os.path.join(
        Path(__file__).resolve(strict=True).parent,
        "noaa_weather_station_list.csv"
    )
)


'''

Building Efficiency Targeting Tool for Energy Retrofits (BETTER) Copyright (c) 2018, The Regents of the University of California, through Lawrence Berkeley National Laboratory (subject to receipt of any required approvals from the U.S. Dept. of Energy). All rights reserved.

If you have questions about your rights to use or distribute this software, please contact Berkeley Lab's Intellectual Property Office at  IPO@lbl.gov.

NOTICE.  This Software was developed
under funding from the U.S. Department of Energy and the U.S. Government consequently retains certain rights. As such, the U.S. Government has been granted for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license in the Software to reproduce, distribute copies to the public, prepare derivative works, and perform publicly and display publicly, and to permit other to do so.

'''

from .constants import Constants
import pandas as pd
import numpy as np
import os
from ish_parser import ish_report, ish_reportException
# from ftplib import FTP
import gzip
import requests

def process_downloaded_weather_noaa(weather_station_ID, start_year, end_year, temperature_unit = "C"):
        # Helper functions
        def download_sub_hourly_weather(station_ID, year):
            # ftp = FTP('ftp.ncdc.noaa.gov')
            # ftp.login()
            # ftp_path = '/pub/data/noaa/' + str(year)  # Hourly
            # ftp.cwd(ftp_path)
            # Save gz files locally
            # file_name_noaa = 'RETR ' + station_ID + '-' + str(year) + '.gz'
            file_name_local = station_ID + '-' + str(year) + '.gz'
            # ftp.retrbinary(file_name_noaa, open(file_name_local, 'wb').write)

            # using requests
            print(f"Process downloading the data for station: {station_ID}")
            url = f"https://www.ncei.noaa.gov/pub/data/noaa/{year}/{station_ID}-{year}.gz"
            response = requests.get(url)
            if response.status_code == 200:
                # Open the local file in binary write mode and write the content
                with open(file_name_local, 'wb') as file:
                    file.write(response.content)
                print(f"File downloaded successfully to {file_name_local}")
            else:
                print(f"Failed to download the file. Status code: {response.status_code}")

        def save_as_csv(stationID, year):
            file_name_gz = stationID + '-' + str(year) + '.gz'
            file_name_csv = stationID + '-' + str(year) + '.csv'
            with gzip.open(file_name_gz, 'rb') as infile:
                with open(file_name_csv, 'wb') as outfile:
                    for line in infile:
                        outfile.write(line)
            os.remove(file_name_gz)

        def get_ish_report_helper(raw_rpt):
            return (ish_report().loads(raw_rpt))

        def get_ish_report_datetime_helper(raw_rpt):
            return (raw_rpt.datetime)

        def get_ish_report_temperature_helper(raw_rpt):
            return (raw_rpt.air_temperature.get_fahrenheit())
        
        for year in range(start_year, end_year + 1):
            print("--->" + str(year))
            download_sub_hourly_weather(weather_station_ID, year)
            save_as_csv(weather_station_ID, year)
            raw_csv = weather_station_ID + '-' + str(year) + '.csv'
            if (year == start_year):
                df_raw = pd.read_csv(raw_csv, names=['Old'])
            else:
                df_raw = pd.concat(
                    [df_raw, pd.read_csv(raw_csv, names=['Old'])],
                    ignore_index=True
                )
            # Cleaning up
            os.remove(raw_csv)
        # Parse ish text data to readable weather data
        print("Processing downloaded data...")
        v_rpt = list(map(get_ish_report_helper, df_raw['Old']))
        v_noaa_datetime = np.array(list(map(get_ish_report_datetime_helper, v_rpt)))
        v_noaa_temperature_F = np.array(list(map(get_ish_report_temperature_helper, v_rpt)))
        # Sanitize the array
        v_noaa_temperature_F = pd.to_numeric(v_noaa_temperature_F, errors='coerce')
        df_new = pd.DataFrame({'Datetime': v_noaa_datetime, 'Temperature': v_noaa_temperature_F})
        df_new = df_new.groupby(
            pd.Grouper(
                key='Datetime',freq='D'
            )
        ).mean()

        if temperature_unit=="F":
            return df_new

        # converting Farenheit to Celcius
        df_new["Temperature"] = (df_new["Temperature"]-32)/1.8

        return df_new


class Weather:

    def __init__(self, coord):
        self.coord = coord
        self.latitude, self.longitude = coord  # geo-coded address
        self.find_closest_weather_station()

    def process(self, df_periods):
        df_periods['start_dates'] = pd.to_datetime(df_periods['start_dates'], utc=True)
        df_periods['end_dates'] = pd.to_datetime(df_periods['end_dates'], utc=True)
        self.df_periods = df_periods
        self.v_start_dates = self.df_periods.loc[:, 'start_dates']
        self.v_end_dates = self.df_periods.loc[:, 'end_dates']
        self.start_year = pd.DatetimeIndex(np.sort(self.v_start_dates)).year[0]
        self.end_year = pd.DatetimeIndex(np.sort(self.v_end_dates)).year[-1]

    @staticmethod
    def haversine_distance(lat1, lon1, lat2, lon2):
        # Get radians from decimals
        r_lat1, r_lon1, r_lat2, r_lon2 = map(np.radians, [lat1, lon1, lat2, lon2])

        # Calculate the distance between the two locations
        temp = np.sin((r_lat2 - r_lat1) / 2) ** 2 + np.cos(r_lat1) * np.cos(r_lat2) * np.sin((r_lon2 - r_lon1) / 2) ** 2
        distance = 2 * Constants.earth_radius * np.arcsin(np.sqrt(temp))
        return (distance)

    def find_closest_weather_station(self, df_weather_station_list=Constants.df_noaa_weather_station):
        self.v_coord = np.asarray(df_weather_station_list[['latitude', 'longitude']].values)
        # Find the closest and second closest weather station (backup if the closest doesn't work)
        v_distance = [Weather.haversine_distance(self.latitude, self.longitude, coord[0], coord[1])
                      for coord in self.v_coord]
        closest_index = np.argmin(v_distance)
        second_closest_index = np.argpartition(v_distance, 2)[2]
        third_closest_index = np.argpartition(v_distance, 3)[2]

        self.closest_weather_station_ID = df_weather_station_list.loc[closest_index, 'station_ID']
        self.closest_weather_station_name = df_weather_station_list.loc[closest_index, 'station_name']
        self.second_closest_weather_station_ID = df_weather_station_list.loc[second_closest_index, 'station_ID']
        self.second_closest_weather_station_name = df_weather_station_list.loc[second_closest_index, 'station_name']
        self.third_closest_weather_station_ID = df_weather_station_list.loc[third_closest_index, 'station_ID']
        self.third_closest_weather_station_name = df_weather_station_list.loc[third_closest_index, 'station_name']

    def download_weather_NOAA(self,temperature_daily_data = None, temperature_unit = "C"):
        if not isinstance(temperature_daily_data,pd.DataFrame) or temperature_daily_data.empty:
            print("Downloading weather data...")
            try:
                self.v_T_F, self.v_T_C = self.process_downloaded_weather(self.closest_weather_station_ID)
            except:
                try:
                    print("Weather from the closest weather station not available...")
                    print("Trying to download the second weather data from the second closest weather station.")
                    self.v_T_F, self.v_T_C = self.process_downloaded_weather(self.second_closest_weather_station_ID)
                except:
                    print("Weather from the second closest weather station not available...")
                    print("Trying to download the third weather data from the third closest weather station.")
                    self.v_T_F, self.v_T_C = self.process_downloaded_weather(self.third_closest_weather_station_ID)
        else:
            if not all([el in temperature_daily_data.columns for el in ["Datetime","Temperature"]]):
                raise Exception(
                    "temperature daily data must contain Datetime and Temperature as column"
                )
            
            if temperature_unit == "C":
                temperature_daily_data.loc[:,"Temperature"] = temperature_daily_data.loc[:,"Temperature"]*1.8+32

            self.v_T_F, self.v_T_C = self.aggregate_weather(temperature_daily_data)

    def use_downloaded_weather(self):
        s_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
        try:
            self.v_T_F, self.v_T_C = self.process_cached_weather(self.closest_weather_station_ID, s_path)
        except:
            try:
                print("Weather from the closest weather station not available...")
                print("Trying to process the third second data from the third closest weather station.")
                self.v_T_F, self.v_T_C = self.process_cached_weather(self.second_closest_weather_station_ID, s_path)
            except:
                print("Weather from the second closest weather station not available...")
                print("Trying to process the third weather data from the third closest weather station.")
                self.v_T_F, self.v_T_C = self.process_cached_weather(self.third_closest_weather_station_ID, s_path)
    
    def process_cached_weather(self, weather_station_ID, s_path):
        for year in range(self.start_year, self.end_year + 1):
            print("Process weather data for year: " + str(year))
            # Read pre-processed weather files from weather file folders
            file_name = (s_path + "/Data/Weather/" + str(year) + "/" +
                         str(year) + "_" + weather_station_ID + '.csv')
    
            if (year == self.start_year):
                df_new = pd.read_csv(file_name)
            else:
                # df_new = df_new.append(pd.read_csv(file_name), ignore_index=True)
                df_new = pd.concat(
                    [df_new, pd.read_csv(file_name)],
                    ignore_index=True
                )
        df_new['Datetime'] = df_new['Datetime'].astype('datetime64[ns]')
        df_new['Date'] = df_new['Datetime'].dt.date
        v_T_F, v_T_C = self.aggregate_weather(df_new)
        return(v_T_F, v_T_C)

    def process_downloaded_weather(self, weather_station_ID):

        # Helper functions
        def download_sub_hourly_weather(station_ID, year):
            # ftp = FTP('ftp.ncdc.noaa.gov')
            # ftp.login()
            # ftp_path = '/pub/data/noaa/' + str(year)  # Hourly
            # ftp.cwd(ftp_path)
            # Save gz files locally
            # file_name_noaa = 'RETR ' + station_ID + '-' + str(year) + '.gz'
            file_name_local = station_ID + '-' + str(year) + '.gz'
            # ftp.retrbinary(file_name_noaa, open(file_name_local, 'wb').write)

            # using requests
            print(f"Process downloading the data for station: {station_ID}")
            url = f"https://www.ncei.noaa.gov/pub/data/noaa/{year}/{station_ID}-{year}.gz"
            response = requests.get(url)
            if response.status_code == 200:
                # Open the local file in binary write mode and write the content
                with open(file_name_local, 'wb') as file:
                    file.write(response.content)
                print(f"File downloaded successfully to {file_name_local}")
            else:
                print(f"Failed to download the file. Status code: {response.status_code}")


        def save_as_csv(stationID, year):
            file_name_gz = stationID + '-' + str(year) + '.gz'
            file_name_csv = stationID + '-' + str(year) + '.csv'
            with gzip.open(file_name_gz, 'rb') as infile:
                with open(file_name_csv, 'wb') as outfile:
                    for line in infile:
                        outfile.write(line)
            os.remove(file_name_gz)

        def get_ish_report_helper(raw_rpt):
            return (ish_report().loads(raw_rpt))

        def get_ish_report_datetime_helper(raw_rpt):
            return (raw_rpt.datetime)

        def get_ish_report_temperature_helper(raw_rpt):
            return (raw_rpt.air_temperature.get_fahrenheit())
        
        for year in range(self.start_year, self.end_year + 1):
            print("--->" + str(year))
            download_sub_hourly_weather(weather_station_ID, year)
            save_as_csv(weather_station_ID, year)
            raw_csv = weather_station_ID + '-' + str(year) + '.csv'
            if (year == self.start_year):
                df_raw = pd.read_csv(raw_csv, names=['Old'])
            else:
                # df_raw = df_raw.append(pd.read_csv(raw_csv, names=['Old']),
                #                        ignore_index=True)
                df_raw = pd.concat(
                    [df_raw, pd.read_csv(raw_csv, names=['Old'])],
                    ignore_index=True
                )
            # Cleaning up
            os.remove(raw_csv)
        # Parse ish text data to readable weather data
        print("Processing downloaded data...")
        v_rpt = list(map(get_ish_report_helper, df_raw['Old']))
        v_noaa_datetime = np.array(list(map(get_ish_report_datetime_helper, v_rpt)))
        v_noaa_temperature_F = np.array(list(map(get_ish_report_temperature_helper, v_rpt)))
        # Sanitize the array
        v_noaa_temperature_F = pd.to_numeric(v_noaa_temperature_F, errors='coerce')
        df_new = pd.DataFrame({'Datetime': v_noaa_datetime, 'Temperature': v_noaa_temperature_F})
        df_new['Date'] = df_new['Datetime'].dt.date
        v_T_F, v_T_C = self.aggregate_weather(df_new)
        return(v_T_F, v_T_C)

    def aggregate_weather(self, df_daily):
        # Get daily weather data

        # Remove time zone information
        self.v_start_dates = np.array(self.v_start_dates, dtype=np.datetime64)
        self.v_end_dates = np.array(self.v_end_dates, dtype=np.datetime64)
        df_daily['Datetime'] = np.array(df_daily['Datetime'], dtype=np.datetime64)

        df_daily = df_daily.loc[(df_daily['Datetime'] >=min(self.v_start_dates)) &
                                (df_daily['Datetime'] <= max(self.v_end_dates))]

        # Aggregate the weather data to the billing periods level.
        v_avg_period_T_F = np.empty(0, dtype=float)
        v_start_dates = np.array(self.v_start_dates)
        v_end_dates = np.array(self.v_end_dates)
        v_temp_datetime = np.array(df_daily['Datetime'], dtype=np.datetime64)
        for i in range(len(self.v_start_dates)):
            df_temp = df_daily.loc[(v_temp_datetime >= v_start_dates[i]) &
                                   (v_temp_datetime < v_end_dates[i])]
            v_avg_period_T_F = np.append(v_avg_period_T_F, df_temp['Temperature'].mean())
        v_avg_period_T_C = (v_avg_period_T_F - 32) / 1.8

        return (v_avg_period_T_F, v_avg_period_T_C)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pipenv install twine --dev

import io
import os

from setuptools import find_packages, setup
from setuptools.command.install import install
from setuptools.command.develop import develop

import berset_engine
# ==============Package meta-data==============
NAME = 'berset_engine'
DESCRIPTION = 'Bulding Energy Retrofit Scenario Evaluation Tool Engine'
URL = 'https://github.com/longlevan/berset_engine'
EMAIL = 'long.le-van@outlook.com'
AUTHOR = 'Van Long Le'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = berset_engine.__version__
STATUS = "Underdevelopment"

# What packages are required for this module to be executed
REQUIRED = [
    'pandas>=2.0.0',
    'geocoder>=1.38.1',
    'ish_parser>=0.0.22',
    'numpy>=1.20.0',
    'scipy>=1.11.0',
    'xlrd>=2.0.1',
    'geopy>=2.4.1',
    'python-dotenv>=1.0.1',
    'plotly>=5.8.0',
    'Jinja2>=3.1.2',
    "python-dotenv>=1.0.1"
]

# What packages are optional
EXTRAS = {
    # 'plotly': ['plotly'],
}

here = os.path.abspath(os.path.dirname(__file__))

# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION


# class PostInstallCommand(install):
#     """
#     Post-installation for installation mode
#     NOTE: Only works when installing a source distribution zip
#     or tarball, or installing in editable mode from a source tree.
#     It will not work when installing from a binary wheel (.whl)
#     """
#     @staticmethod
#     def status(s):
#         """Prints things in bold."""
#         print('\033[1m{0}\033[0m'.format(s))

#     def run(self):
#         install.run(self)
#         self.status("run config scripts")
#         os.system('berset-config')


# class PostDevelopCommand(develop):
#     @staticmethod
#     def status(s):
#         """Prints things in bold."""
#         print('\033[1m{0}\033[0m'.format(s))

#     def run(self):
#         develop.run(self)
#         self.status("run config scripts")
#         os.system('berset-config')


setup(
    name=NAME,
    version=about['__version__'],
    license='MIT',
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    packages=find_packages(
        exclude=[
            "tests",
            "*.tests",
            "*.tests.*",
            "tests.*"
        ]
    ),
    # package_dir = {},
    # python_modules=[],
    package_data={"": ["*.cfg"]},
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    project_urls={
        "Documentation": "https://github.com/longlevan/berset_engine/",
        "Changelog": (
            "https://github.com/longlevan/berset_engine/CHANGELOG.md"
        ),
        "Issue Tracker": "https://github.com/longlevan/berset_engine/issues",
    },
    keywords=[
        # eg: 'keyword1'
    ],
    python_requires=REQUIRES_PYTHON,
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    entry_points={
        # 'console_scripts': [
        #     'berset=berset_engine.interfaces.cli:main',
        # ],
    },
    # cmdclass={
    #     'install': PostInstallCommand,
    #     'develop': PostDevelopCommand
    # }
)
